@@ -58,7 +58,13 @@
 #include "conf.h"
 #include "uioctl.h"
 
+#if FUSE_USE_VERSION < 30
 static int unionfs_chmod(const char *path, mode_t mode) {
+#else
+static int unionfs_chmod(const char *path, mode_t mode, struct fuse_file_info *fi) {
+	(void) fi;  // just to prevent the compiler complaining about unused variables
+#endif
+
 	DBG("%s\n", path);
 
 	int i = find_rw_branch_cow(path);
@@ -73,7 +79,13 @@ static int unionfs_chmod(const char *path, mode_t mode) {
 	RETURN(0);
 }
 
+#if FUSE_USE_VERSION < 30
 static int unionfs_chown(const char *path, uid_t uid, gid_t gid) {
+#else
+static int unionfs_chown(const char *path, uid_t uid, gid_t gid, struct fuse_file_info *fi) {
+	(void) fi;  // just to prevent the compiler complaining about unused variables
+#endif
+
 	DBG("%s\n", path);
 
 	int i = find_rw_branch_cow(path);
@@ -127,6 +139,8 @@ static int unionfs_create(const char *path, mode_t mode, struct fuse_file_info *
  * which flush the data/metadata on close()
  */
 static int unionfs_flush(const char *path, struct fuse_file_info *fi) {
+	(void) path;  // just to prevent the compiler complaining about unused variables
+
 	DBG("fd = %"PRIx64"\n", fi->fh);
 
 	int fd = dup(fi->fh);
@@ -148,6 +162,8 @@ static int unionfs_flush(const char *path, struct fuse_file_info *fi) {
  * Just a stub. This method is optional and can safely be left unimplemented
  */
 static int unionfs_fsync(const char *path, int isdatasync, struct fuse_file_info *fi) {
+	(void) path;  // just to prevent the compiler complaining about unused variables
+
 	DBG("fd = %"PRIx64"\n", fi->fh);
 
 	int res;
@@ -166,7 +182,13 @@ static int unionfs_fsync(const char *path, int isdatasync, struct fuse_file_info
 	RETURN(0);
 }
 
+#if FUSE_USE_VERSION < 30
 static int unionfs_getattr(const char *path, struct stat *stbuf) {
+#else
+static int unionfs_getattr(const char *path, struct stat *stbuf, struct fuse_file_info *fi) {
+	(void) fi;  // just to prevent the compiler complaining about unused variables
+#endif
+
 	DBG("%s\n", path);
 
 	int i = find_rorw_branch(path);
@@ -193,7 +215,11 @@ static int unionfs_getattr(const char *path, struct stat *stbuf) {
 static int unionfs_access(const char *path, int mask) {
 	struct stat s;
 
+#if FUSE_USE_VERSION < 30
 	if (unionfs_getattr(path, &s) != 0)
+#else
+	if (unionfs_getattr(path, &s, NULL) != 0)
+#endif
 		RETURN(-ENOENT);
 
 	if ((mask & X_OK) && (s.st_mode & S_IXUSR) == 0)
@@ -212,9 +238,15 @@ static int unionfs_access(const char *path, int mask) {
  * init method
  * called before first access to the filesystem
  */
-static void * unionfs_init(struct fuse_conn_info *conn) {
-	// just to prevent the compiler complaining about unused variables
-	(void) conn->max_readahead;
+#if FUSE_USE_VERSION < 30
+static void *unionfs_init(struct fuse_conn_info *conn) {
+#else
+static void *unionfs_init(struct fuse_conn_info *conn, struct fuse_config *cfg) {
+	(void) cfg;  // just to prevent the compiler complaining about unused variables
+#endif
+	(void) conn;  // just to prevent the compiler complaining about unused variables
+
+	init_syslog();
 
 	// we only now (from unionfs_init) may go into the chroot, since otherwise
 	// fuse_main() will fail to open /dev/fuse and to call mount
@@ -260,8 +292,11 @@ static int unionfs_link(const char *from, const char *to) {
 	RETURN(0);
 }
 
-#if FUSE_VERSION >= 28
+#if FUSE_USE_VERSION < 35
 static int unionfs_ioctl(const char *path, int cmd, void *arg, struct fuse_file_info *fi, unsigned int flags, void *data) {
+#else
+static int unionfs_ioctl(const char *path, unsigned int cmd, void *arg, struct fuse_file_info *fi, unsigned int flags, void *data) {
+#endif
 	(void) path;
 	(void) arg; // avoid compiler warning
 	(void) fi;  // avoid compiler warning
@@ -298,7 +333,6 @@ static int unionfs_ioctl(const char *path, int cmd, void *arg, struct fuse_file_
 
 	return 0;
 }
-#endif
 
 /**
  * unionfs mkdir() implementation
@@ -396,6 +430,8 @@ static int unionfs_open(const char *path, struct fuse_file_info *fi) {
 }
 
 static int unionfs_read(const char *path, char *buf, size_t size, off_t offset, struct fuse_file_info *fi) {
+	(void) path;  // just to prevent the compiler complaining about unused variables
+
 	DBG("fd = %"PRIx64"\n", fi->fh);
 
 	int res = pread(fi->fh, buf, size, offset);
@@ -424,6 +460,8 @@ static int unionfs_readlink(const char *path, char *buf, size_t size) {
 }
 
 static int unionfs_release(const char *path, struct fuse_file_info *fi) {
+	(void) path;  // just to prevent the compiler complaining about unused variables
+
 	DBG("fd = %"PRIx64"\n", fi->fh);
 
 	int res = close(fi->fh);
@@ -437,9 +475,16 @@ static int unionfs_release(const char *path, struct fuse_file_info *fi) {
  * TODO: If we rename a directory on a read-only branch, we need to copy over
  *       all files to the renamed directory on the read-write branch.
  */
+#if FUSE_USE_VERSION < 30
 static int unionfs_rename(const char *from, const char *to) {
+#else
+static int unionfs_rename(const char *from, const char *to, unsigned int flags) {
+	(void) flags;  // just to prevent the compiler complaining about unused variables
+#endif
+
 	DBG("from %s to %s\n", from, to);
 
+	int res;
 	bool is_dir = false; // is 'from' a file or directory
 
 	int j = find_rw_branch_cutlast(to);
@@ -448,6 +493,28 @@ static int unionfs_rename(const char *from, const char *to) {
 	int i = find_rorw_branch(from);
 	if (i == -1) RETURN(-errno);
 
+	if (uopt.preserve_branch && uopt.branches[i].rw) {
+		int existing = find_rorw_branch(to);
+
+		if (existing != -1 && existing != i) {
+			USYSLOG(LOG_ERR, "%s: from %s would overwrite to on a different branch, which"
+				"is not supported.\n", __func__, from);
+			RETURN(-EXDEV);
+		}
+
+		if (branch_contains_file_or_parent_dir(i, to)) {
+			DBG("preserving branch\n");
+			j = i;
+		} else {
+			DBG("preserving branch and creating directories to do so\n");
+			res = path_create_cutlast(to, j, i);
+
+			if (res == 0) {
+				j = i;
+			}
+		}
+	}
+
 	if (!uopt.branches[i].rw) {
 		i = find_rw_branch_cow_common(from, true);
 		if (i == -1) RETURN(-errno);
@@ -455,7 +522,7 @@ static int unionfs_rename(const char *from, const char *to) {
 
 	if (i != j) {
 		USYSLOG(LOG_ERR, "%s: from and to are on different writable branches %d vs %d, which"
-		       "is not supported yet.\n", __func__, i, j);
+			"is not supported yet.\n", __func__, i, j);
 		RETURN(-EXDEV);
 	}
 
@@ -464,19 +531,20 @@ static int unionfs_rename(const char *from, const char *to) {
 	if (BUILD_PATH(t, uopt.branches[i].path, to)) RETURN(-ENAMETOOLONG);
 
 	filetype_t ftype = path_is_dir(f);
-	if (ftype == NOT_EXISTING)
+	if (ftype == NOT_EXISTING) {
 		RETURN(-ENOENT);
-	else if (ftype == IS_DIR)
+	} else if (ftype == IS_DIR) {
 		is_dir = true;
+	}
 
-	int res;
 	if (!uopt.branches[i].rw) {
 		// since original file is on a read-only branch, we copied the from file to a writable branch,
 		// but since we will rename from, we also need to hide the from file on the read-only branch
-		if (is_dir)
+		if (is_dir) {
 			res = hide_dir(from, i);
-		else
+		} else {
 			res = hide_file(from, i);
+		}
 		if (res) RETURN(-errno);
 	}
 
@@ -486,13 +554,15 @@ static int unionfs_rename(const char *from, const char *to) {
 		int err = errno; // unlink() might overwrite errno
 		// if from was on a read-only branch we copied it, but now rename failed so we need to delete it
 		if (!uopt.branches[i].rw) {
-			if (unlink(f))
+			if (unlink(f)) {
 				USYSLOG(LOG_ERR, "%s: cow of %s succeeded, but rename() failed and now "
-				       "also unlink()  failed\n", __func__, from);
+					"also unlink()  failed\n", __func__, from);
+			}
 
-			if (remove_hidden(from, i))
+			if (remove_hidden(from, i)) {
 				USYSLOG(LOG_ERR, "%s: cow of %s succeeded, but rename() failed and now "
-				       "also removing the whiteout  failed\n", __func__, from);
+					"also removing the whiteout  failed\n", __func__, from);
+			}
 		}
 		RETURN(-err);
 	}
@@ -501,10 +571,11 @@ static int unionfs_rename(const char *from, const char *to) {
 		// A lower branch still *might* have a file called 'from', we need to delete this.
 		// We only need to do this if we have been on a rw-branch, since we created
 		// a whiteout for read-only branches anyway.
-		if (is_dir)
+		if (is_dir) {
 			maybe_whiteout(from, i, WHITEOUT_DIR);
-		else
+		} else {
 			maybe_whiteout(from, i, WHITEOUT_FILE);
+		}
 	}
 
 	remove_hidden(to, i); // remove hide file (if any)
@@ -651,7 +722,13 @@ static int unionfs_symlink(const char *from, const char *to) {
 	RETURN(0);
 }
 
+#if FUSE_USE_VERSION < 30
 static int unionfs_truncate(const char *path, off_t size) {
+#else
+static int unionfs_truncate(const char *path, off_t size, struct fuse_file_info *fi) {
+	(void) fi;  // just to prevent the compiler complaining about unused variables
+#endif
+
 	DBG("%s\n", path);
 
 	int i = find_rw_branch_cow(path);
@@ -667,7 +744,13 @@ static int unionfs_truncate(const char *path, off_t size) {
 	RETURN(0);
 }
 
+#if FUSE_USE_VERSION < 30
 static int unionfs_utimens(const char *path, const struct timespec ts[2]) {
+#else
+static int unionfs_utimens(const char *path, const struct timespec ts[2], struct fuse_file_info *fi) {
+	(void) fi;  // just to prevent the compiler complaining about unused variables
+#endif
+
 	DBG("%s\n", path);
 
 	int i = find_rw_branch_cow(path);
@@ -803,9 +886,7 @@ struct fuse_operations unionfs_oper = {
 	.getattr = unionfs_getattr,
 	.access = unionfs_access,
 	.init = unionfs_init,
-#if FUSE_VERSION >= 28
 	.ioctl = unionfs_ioctl,
-#endif
 	.link = unionfs_link,
 	.mkdir = unionfs_mkdir,
 	.mknod = unionfs_mknod,
