@@ -24,6 +24,7 @@
 static struct fuse_opt unionfs_opts[] = {
 	FUSE_OPT_KEY("chroot=%s,", KEY_CHROOT),
 	FUSE_OPT_KEY("cow", KEY_COW),
+	FUSE_OPT_KEY("preserve_branch", KEY_PRESERVE_BRANCH),
 	FUSE_OPT_KEY("debug_file=%s", KEY_DEBUG_FILE),
 	FUSE_OPT_KEY("dirs=%s", KEY_DIRS),
 	FUSE_OPT_KEY("--help", KEY_HELP),
@@ -42,7 +43,8 @@ static struct fuse_opt unionfs_opts[] = {
 int main(int argc, char *argv[]) {
 	struct fuse_args args = FUSE_ARGS_INIT(argc, argv);
 
-	init_syslog();
+	// Initing syslog here is a problem because the log thread ends up exiting in non foreground mode (see https://github.com/rpodgorny/unionfs-fuse/issues/122)
+	// init_syslog();
 	uopt_init();
 
 	if (fuse_opt_parse(&args, NULL, unionfs_opts, unionfs_opt_proc) == -1) RETURN(1);
